@@ -15,8 +15,6 @@ from ormar.models import NewBaseModel  # noqa I100
 from ormar.models.metaclass import ModelMeta
 from ormar.models.model_row import ModelRow
 
-if TYPE_CHECKING:  # pragma nocover
-    from ormar import QuerySet
 
 T = TypeVar("T", bound="Model")
 
@@ -25,7 +23,6 @@ class Model(ModelRow):
     __abstract__ = False
     if TYPE_CHECKING:  # pragma nocover
         Meta: ModelMeta
-        objects: "QuerySet"
 
     def __repr__(self) -> str:  # pragma nocover
         _repr = {k: getattr(self, k) for k, v in self.Meta.model_fields.items()}
